@@ -92,11 +92,20 @@ static void     gdm_simple_slave_finalize       (GObject             *object);
 
 G_DEFINE_TYPE (GdmSimpleSlave, gdm_simple_slave, GDM_TYPE_SLAVE)
 
+static void create_new_session (GdmSimpleSlave *slave);
+
 static void
 on_session_started (GdmSession       *session,
+                    int               pid,
                     GdmSimpleSlave   *slave)
 {
-        g_debug ("GdmSimpleSlave: session started");
+        char *username;
+
+        g_debug ("GdmSimpleSlave: session started %d", pid);
+
+        /* Run the PreSession script. gdmslave suspends until script has terminated */
+        username = gdm_session_direct_get_username (slave->priv->session);
+        gdm_slave_run_script (GDM_SLAVE (slave), GDMCONFDIR "/PreSession", username);
 
         /* FIXME: should we do something here? */
 }
@@ -106,8 +115,14 @@ on_session_exited (GdmSession     *session,
                    int             exit_code,
                    GdmSimpleSlave *slave)
 {
+        char *username;
+
         g_debug ("GdmSimpleSlave: session exited with code %d\n", exit_code);
 
+        /* Run the PostSession script. gdmslave suspends until script has terminated */
+        username = gdm_session_direct_get_username (slave->priv->session);
+        gdm_slave_run_script (GDM_SLAVE (slave), GDMCONFDIR "/PostSession", username);
+
         gdm_slave_stopped (GDM_SLAVE (slave));
 }
 
@@ -139,10 +154,30 @@ add_user_authorization (GdmSimpleSlave *slave,
         return ret;
 }
 
+static void
+destroy_session (GdmSimpleSlave *slave)
+{
+        if (slave->priv->session != NULL) {
+                gdm_session_close (GDM_SESSION (slave->priv->session));
+                g_object_unref (slave->priv->session);
+                slave->priv->session = NULL;
+        }
+}
+
+static void
+reset_session (GdmSimpleSlave *slave)
+{
+        destroy_session (slave);
+        create_new_session (slave);
+        gdm_session_open (GDM_SESSION (slave->priv->session));
+}
+
 static gboolean
 greeter_reset_timeout (GdmSimpleSlave *slave)
 {
+        g_debug ("GdmSimpleSlave: resetting greeter");
         gdm_greeter_server_reset (slave->priv->greeter_server);
+        reset_session (slave);
         slave->priv->greeter_reset_id = 0;
         return FALSE;
 }
@@ -171,6 +206,7 @@ on_session_setup_failed (GdmSession     *session,
 {
         gdm_greeter_server_problem (slave->priv->greeter_server, _("Unable to initialize login system"));
 
+        destroy_session (slave);
         queue_greeter_reset (slave);
 }
 
@@ -202,7 +238,7 @@ on_session_authentication_failed (GdmSession     *session,
                                   GdmSimpleSlave *slave)
 {
         gdm_greeter_server_problem (slave->priv->greeter_server, _("Unable to authenticate user"));
-
+        destroy_session (slave);
         queue_greeter_reset (slave);
 }
 
@@ -210,8 +246,25 @@ static void
 gdm_simple_slave_accredit_when_ready (GdmSimpleSlave *slave)
 {
         if (slave->priv->start_session_when_ready) {
-                gdm_session_accredit (GDM_SESSION (slave->priv->session),
-                                      GDM_SESSION_CRED_ESTABLISH);
+                char *ssid;
+                char *username;
+                int   cred_flag;
+
+                slave->priv->waiting_to_start_session = FALSE;
+
+                username = gdm_session_direct_get_username (slave->priv->session);
+
+                ssid = gdm_slave_get_primary_session_id_for_user (GDM_SLAVE (slave), username);
+                if (ssid != NULL && ssid [0] != '\0') {
+                        /* FIXME: we don't yet support refresh */
+                        cred_flag = GDM_SESSION_CRED_ESTABLISH;
+                } else {
+                        cred_flag = GDM_SESSION_CRED_ESTABLISH;
+                }
+                g_free (ssid);
+                g_free (username);
+
+                gdm_session_accredit (GDM_SESSION (slave->priv->session), cred_flag);
         } else {
                 slave->priv->waiting_to_start_session = TRUE;
         }
@@ -232,6 +285,7 @@ on_session_authorization_failed (GdmSession     *session,
 {
         gdm_greeter_server_problem (slave->priv->greeter_server, _("Unable to authorize user"));
 
+        destroy_session (slave);
         queue_greeter_reset (slave);
 }
 
@@ -265,7 +319,14 @@ start_session_timeout (GdmSimpleSlave *slave)
         migrated = try_migrate_session (slave);
         g_debug ("GdmSimpleSlave: migrated: %d", migrated);
         if (migrated) {
+                destroy_session (slave);
+
+                /* We don't stop the slave here because
+                   when Xorg exits it switches to the VT it was
+                   started from.  That interferes with fast
+                   user switching. */
                 queue_greeter_reset (slave);
+
                 goto out;
         }
 
@@ -304,6 +365,9 @@ on_session_accredited (GdmSession     *session,
                        GdmSimpleSlave *slave)
 {
         queue_start_session (slave);
+
+        /* Run the PostLogin script. gdmslave suspends until script has terminated */
+        gdm_slave_run_script (GDM_SLAVE (slave), GDMCONFDIR "/PostLogin", GDM_USERNAME);
 }
 
 static void
@@ -323,6 +387,12 @@ on_session_accreditation_failed (GdmSession     *session,
                 gdm_greeter_server_problem (slave->priv->greeter_server, _("Unable establish credentials"));
         }
 
+        /* We don't stop the slave here after migrating because
+           when Xorg exits it switches to the VT it was
+           started from.  That interferes with fast
+           user switching. */
+        destroy_session (slave);
+
         queue_greeter_reset (slave);
 }
 
@@ -429,6 +499,16 @@ on_default_language_name_changed (GdmSession     *session,
         gdm_greeter_server_default_language_name_changed (slave->priv->greeter_server, text);
 }
 
+static void
+on_default_layout_name_changed (GdmSession     *session,
+                                const char     *text,
+                                GdmSimpleSlave *slave)
+{
+        g_debug ("GdmSimpleSlave: Default layout name changed: %s", text);
+
+        gdm_greeter_server_default_layout_name_changed (slave->priv->greeter_server, text);
+}
+
 static void
 on_default_session_name_changed (GdmSession     *session,
                                  const char     *text,
@@ -563,6 +643,11 @@ create_new_session (GdmSimpleSlave *slave)
                           G_CALLBACK (on_default_language_name_changed),
                           slave);
 
+        g_signal_connect (slave->priv->session,
+                          "default-layout-name-changed",
+                          G_CALLBACK (on_default_layout_name_changed),
+                          slave);
+
         g_signal_connect (slave->priv->session,
                           "default-session-name-changed",
                           G_CALLBACK (on_default_session_name_changed),
@@ -657,6 +742,14 @@ on_greeter_language_selected (GdmGreeterServer *greeter_server,
         gdm_session_select_language (GDM_SESSION (slave->priv->session), text);
 }
 
+static void
+on_greeter_layout_selected (GdmGreeterServer *greeter_server,
+                            const char       *text,
+                            GdmSimpleSlave   *slave)
+{
+        gdm_session_select_layout (GDM_SESSION (slave->priv->session), text);
+}
+
 static void
 on_greeter_user_selected (GdmGreeterServer *greeter_server,
                           const char       *text,
@@ -670,15 +763,7 @@ on_greeter_cancel (GdmGreeterServer *greeter_server,
                    GdmSimpleSlave   *slave)
 {
         g_debug ("GdmSimpleSlave: Greeter cancelled");
-
-        if (slave->priv->session != NULL) {
-                gdm_session_close (GDM_SESSION (slave->priv->session));
-                g_object_unref (slave->priv->session);
-        }
-
-        create_new_session (slave);
-
-        gdm_session_open (GDM_SESSION (slave->priv->session));
+        reset_session (slave);
 }
 
 static void
@@ -687,7 +772,7 @@ on_greeter_connected (GdmGreeterServer *greeter_server,
 {
         gboolean display_is_local;
 
-        g_debug ("GdmSimpleSlave: Greeter started");
+        g_debug ("GdmSimpleSlave: Greeter connected");
 
         gdm_session_open (GDM_SESSION (slave->priv->session));
 
@@ -721,8 +806,6 @@ on_start_session_later (GdmGreeterServer *session,
         slave->priv->start_session_when_ready = FALSE;
 }
 
-
-
 static void
 setup_server (GdmSimpleSlave *slave)
 {
@@ -772,7 +855,7 @@ run_greeter (GdmSimpleSlave *slave)
         }
 
         /* Run the init script. gdmslave suspends until script has terminated */
-        gdm_slave_run_script (GDM_SLAVE (slave), GDMCONFDIR "/Init/Default", "gdm");
+        gdm_slave_run_script (GDM_SLAVE (slave), GDMCONFDIR "/Init", GDM_USERNAME);
 
         create_new_session (slave);
 
@@ -801,6 +884,10 @@ run_greeter (GdmSimpleSlave *slave)
                           "language-selected",
                           G_CALLBACK (on_greeter_language_selected),
                           slave);
+        g_signal_connect (slave->priv->greeter_server,
+                          "layout-selected",
+                          G_CALLBACK (on_greeter_layout_selected),
+                          slave);
         g_signal_connect (slave->priv->greeter_server,
                           "user-selected",
                           G_CALLBACK (on_greeter_user_selected),
@@ -931,8 +1018,18 @@ gdm_simple_slave_run (GdmSimpleSlave *slave)
          * exist */
         if (display_is_local) {
                 gboolean res;
+                gboolean disable_tcp;
 
                 slave->priv->server = gdm_server_new (display_name, auth_file);
+
+                disable_tcp = TRUE;
+                if (gdm_settings_client_get_boolean (GDM_KEY_DISALLOW_TCP,
+                                                     &disable_tcp)) {
+                        g_object_set (slave->priv->server,
+                                      "disable-tcp", disable_tcp,
+                                      NULL);
+                }
+
                 g_signal_connect (slave->priv->server,
                                   "exited",
                                   G_CALLBACK (on_server_exited),
